#ifndef __MCD__
#define __MCD__

#define MCD_REPRESENTATION_NULL 0
#define MCD_REPRESENTATION_EMB 1
#define MCD_REPRESENTATION_VOCAB 2
#define MCD_REPRESENTATION_INT 3


#define MCD_INVALID_VALUE -1


#include "dico.h"
#include "feat_types.h"
#include "word_emb.h"
#include "dico_vec.h"

#define mcd_get_dico_label(m) (m)->dico_array[FEAT_TYPE_LABEL]

#define mcd_get_form_col(m) (m)->type[FEAT_TYPE_FORM]
#define mcd_set_form_col(m, v) (m)->type[FEAT_TYPE_FORM] = (v)

/* mcd (multi column description) files describe the format of corpus files */
/* every line of an mcd file describes the content of a column of the corpus file */
/* every line contains four fields separated by a space character */
/* first field is the index of the column described (first column corresponds to index zero) */
/* second field is the name of the column. Such must be taken from the following list: */
/* INDEX, FORM, LEMMA, CPOS, POS, FEAT, LABEL, STAG, INT, GOV, A ... Z */
/* third field correspond to the internal representation of the tokens found in the column described. Four values are possible : */
/* VOCAB if the internal representation is an integer code corresponding to the token */
/* INT if the token is already an integer and its corresponding internal value is the same integer */
/* EMB if the internal representation of the token is a real valued vector. */
/* _   if no internal representation is associated to the field */
/* fourth field is the name of a file in which the encoding is represented, this file can either be a dico (see dico.h) format file or an embedding file (see word_emb.h)*/

typedef struct {
  int nb_col;                 /* number of columns in the mcd file */
  int type2col[FEAT_TYPE_NB]; /* in which column is represented is the form (FEAT_TYPE_FORM) lemma ... represented */ 
  /* int *col2type; */
  int *type;                  /* array containing the type of every column */
  char **type_str;            /* a string version of array type */
  int *representation;        /* array containing the representation mode of every column (integer, vocabulary, embedding, NULL) */
  char **filename;            /* array containing the file in which the different values for a columnn is represented */
  dico **dico_array;          /* array containing the dico corresponding to each column (NULL if no file) */
  word_emb **word_emb_array;  /* array containing the word embedding structure corresponding to each column (NULL if no file) */
} mcd;

mcd *mcd_build_conll07(void);
mcd *mcd_build_ifpls(void);

mcd *mcd_read(char *mcd_filename, int verbose);
void mcd_link_to_dico(mcd *m, dico_vec *vocabs, int verbose);
void mcd_extract_dico_from_corpus(mcd *m, char *corpus_filename);
void mcd_free(mcd *m);
int mcd_get_code(mcd *m, char *str, int col);
dico_vec *mcd_build_dico_vec(mcd *mcd_struct);

#endif
