#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<ctype.h>
#include"word.h"
#include"util.h"
#include"feat_types.h"

word *word_new(char *input)
{
  int i;
  word *w = (word *) memalloc(sizeof(word));
  if(input == NULL)
    w->input = NULL;
  else
    w->input = strdup(input);
  
  for(i=0; i < FEAT_TYPE_NB; i++) w->feat_array[i] = -1;
  w->form = NULL;
  return w;
}


/* look for the next word in file f */
word *word_read(FILE *f, mcd *mcd_struct)
{
  char buffer[10000];
  if(feof(f)) return NULL; /* no more words to read */

  /* look for a valid word */
  while(fgets(buffer, 10000, f)){
    if(feof(f)) return NULL; /* no more words to read */
    if((buffer[0] != '\n') && (buffer[0] != ' ')){
      /* printf("word = %s\n", buffer); */
      return word_parse_buffer(buffer, mcd_struct);
    }
  }
  return NULL;
}

/* parse string buffer to extract the different word features */
/* codes of the word features are stored in feat_array */

word *word_parse_buffer(char *buffer, mcd *mcd_struct)
{
  char *token;   
  word *w = NULL;
  int column_nb = 0;

  /* remove newline from buffer */
  if(buffer[strlen(buffer)-1] == '\n')  buffer[strlen(buffer)-1] = '\0';  

  w = word_new(buffer);
  token = strtok(buffer, "\t");
  do{
    if((column_nb < mcd_struct->nb_col) &&  (mcd_struct->type[column_nb] != -1)){
      w->feat_array[mcd_struct->type[column_nb]] = mcd_get_code(mcd_struct, token, column_nb);
    }
    if(mcd_struct->type[column_nb] == FEAT_TYPE_FORM){
      w->form = strdup(token);
      w->U1 = isupper(token[0]) ? 1 : 0;
    }
    column_nb++;
  } while((token = strtok(NULL , "\t")));
  
  return w;
}


word *word_copy(word *w)
{
  word *copy = word_new(w->input);
  int i;

  for(i=0; i < FEAT_TYPE_NB; i++)
    copy->feat_array[i] = w->feat_array[i];

  return copy;
}

void word_free(word *w)
{
  if(w == NULL) return;
  if(w->input) free(w->input);
  if(w->form) free(w->form);
  free(w);
}

word *word_create_dummy(mcd *mcd_struct)
{
  word *w = word_new(NULL);
  /* int type; */

  w->feat_array[FEAT_TYPE_INDEX] = 0;

  /*  for(type = 1; type < FEAT_TYPE_NB; type++)
      w->feat_array[type] = -1;*/
    /*    if(mcd_struct->type2col[type] != -1)
	  w->feat_array[type] = mcd_get_code(mcd_struct, (char *) "ROOT", mcd_struct->type2col[type]);*/
  
  return w;
}

void word_print(FILE *f, word *w, mcd *mcd_struct, dico *dico_labels)
{
  if(w == NULL) return;

  fprintf(f, "%s", w->input);
  fprintf(f, "\t%d", word_get_gov(w));
  if(dico_labels)
    fprintf(f, "\t%s", dico_int2string(dico_labels, w->label));
  else
    fprintf(f, "\t%d", word_get_label(w));
}


