#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<unistd.h>
#include<getopt.h>
#include"movement_parser.h"
#include"oracle_parser.h"
#include"feature_table.h"
#include"dico.h"
#include"perceptron.h"
#include"context.h"
#include"cf_file.h"

void cff_cutoff_help_message(context *ctx)
{
  context_general_help_message(ctx);
  fprintf(stderr, "INPUT\n");
  context_cutoff_help_message(ctx);
  context_cff_help_message(ctx);
  fprintf(stderr, "INPUT/OUTPUT\n");
}

void cff_cutoff_check_options(context *ctx)
{
  if(ctx->help
     || !ctx->vocabs_filename
     || !ctx->cff_filename
     ){
    cff_cutoff_help_message(ctx);
    exit(1);
  }
}

int main(int argc, char *argv[])
{
  char buffer[10000];
  FILE *f;
  char *token;
  int n_feat = 0;
  int *occ_table = NULL;
  int occ_removed = 0;
  int feat_removed = 0;
  int i;
  int f_occ = 0;
  int *old2new;
  int feat;

  dico *old_d_feat;
  dico *new_d_feat;
  context *ctx;

  ctx = context_read_options(argc, argv);
  cff_cutoff_check_options(ctx);

  ctx->vocabs = dico_vec_read(ctx->vocabs_filename, ctx->hash_ratio);
  ctx->d_perceptron_features = dico_vec_get_dico(ctx->vocabs, (char *)"d_perceptron_features");

  old_d_feat = ctx->d_perceptron_features;
  new_d_feat = dico_new((char *)"d_perceptron_features", old_d_feat->nbelem * ctx->hash_ratio);
  occ_table = count_occ_of_features(ctx->cff_filename, &n_feat);
  f = myfopen(ctx->cff_filename, "r");
  old2new = (int *)memalloc(n_feat * sizeof(int));

  for(i=0; i < n_feat; i++)
    if(occ_table[i] < ctx->feature_cutoff)
      old2new[i] = -1;
    else
      old2new[i] = dico_add(new_d_feat, dico_int2string(old_d_feat, i));

  feat_removed = old_d_feat->nbelem - new_d_feat->nbelem;

  while(fgets(buffer, 10000, f)){
    buffer[strlen(buffer) - 1] = '\0';
    token = strtok(buffer, "\t");
    printf("%s", token);
    while((token = strtok(NULL, "\t"))){
      f_occ++;
      feat = atoi(token);
       printf("\t%d", old2new[feat]); 
       if(old2new[feat] == -1)       occ_removed++; 
       /*       if(old2new[feat] != -1)
	printf("\t%d", old2new[feat]);
      else
      occ_removed++;*/
    }
    printf("\n");
  }
  fclose(f);


  fprintf(stderr, "total number of features    : %d\n", n_feat);
  fprintf(stderr, "number of features  removed : %d\n", feat_removed);
  fprintf(stderr, "ratio                       : %.3f\n\n", (float)feat_removed / n_feat);

  fprintf(stderr, "total number of feature occurrences : %d\n", f_occ);
  fprintf(stderr, "feature occurrences removed         : %d\n", occ_removed);
  fprintf(stderr, "ratio                               : %.3f\n", (float)occ_removed / f_occ);


  dico_vec_replace_dico(ctx->vocabs, old_d_feat, new_d_feat);
  dico_vec_print(ctx->vocabs_filename, ctx->vocabs);


  /* dico_print(ctx->perceptron_features_filename, new_d_feat); */


  dico_free(new_d_feat);
  free(old2new);

  
  context_free(ctx);
  return 0;
}
