#include "feat_lib.h"
#include "feat_types.h"
#include "feat_fct.h"

void feat_lib_add(feat_lib *fl, int feature_type, char *feature_name, feat_fct feature_fct)
{
  int feature_nb = dico_add(fl->d_features, feature_name); 
  fl->array = (simple_feat_desc **)realloc(fl->array, (feature_nb + 1) * sizeof(simple_feat_desc *));
  fl->array[feature_nb] = simple_feat_desc_new(feature_name, feature_type, feature_fct);
}

simple_feat_desc *feat_lib_get_simple_feat_desc(feat_lib *fl, char *feat_name)
{
  int index = dico_string2int(fl->d_features, feat_name);
  if(index == -1){
    fprintf(stderr, "features function %s unknown\n", feat_name);
    return NULL;
  }
  return fl->array[index];
}

feat_lib *feat_lib_new(void)
{
  feat_lib *fl = (feat_lib *)memalloc(sizeof(feat_lib));
  fl->nbelem = 0;
  fl->d_features = dico_new(NULL, 100);
  fl->array = NULL;
  return fl;
}


feat_lib *feat_lib_build(void)
{
  feat_lib *fl = feat_lib_new();
  
  feat_lib_add(fl, FEAT_TYPE_FORM,  (char *)"s0f", s0f);
  feat_lib_add(fl, FEAT_TYPE_LEMMA, (char *)"s0l", s0l);
  feat_lib_add(fl, FEAT_TYPE_CPOS,  (char *)"s0c", s0c);
  feat_lib_add(fl, FEAT_TYPE_POS,   (char *)"s0p", s0p);
  feat_lib_add(fl, FEAT_TYPE_FEATS, (char *)"s0m", s0m);
  feat_lib_add(fl, FEAT_TYPE_STAG , (char *)"s0s", s0s);
  /* feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"s0r", s0r); */
  feat_lib_add(fl, FEAT_TYPE_A,     (char *)"s0A", s0A);
  feat_lib_add(fl, FEAT_TYPE_B,     (char *)"s0B", s0B);
  feat_lib_add(fl, FEAT_TYPE_C,     (char *)"s0C", s0C);
  feat_lib_add(fl, FEAT_TYPE_D,     (char *)"s0D", s0D);
  feat_lib_add(fl, FEAT_TYPE_E,     (char *)"s0E", s0E);
  feat_lib_add(fl, FEAT_TYPE_F,     (char *)"s0F", s0F);
  feat_lib_add(fl, FEAT_TYPE_G,     (char *)"s0G", s0G);
  feat_lib_add(fl, FEAT_TYPE_H,     (char *)"s0H", s0H);
  feat_lib_add(fl, FEAT_TYPE_I,     (char *)"s0I", s0I);
  feat_lib_add(fl, FEAT_TYPE_J,     (char *)"s0J", s0J);
  feat_lib_add(fl, FEAT_TYPE_K,     (char *)"s0K", s0K);
  feat_lib_add(fl, FEAT_TYPE_L,     (char *)"s0L", s0L);
  feat_lib_add(fl, FEAT_TYPE_M,     (char *)"s0M", s0M);
  feat_lib_add(fl, FEAT_TYPE_N,     (char *)"s0N", s0N);
  feat_lib_add(fl, FEAT_TYPE_O,     (char *)"s0O", s0O);
  feat_lib_add(fl, FEAT_TYPE_P,     (char *)"s0P", s0P);
  feat_lib_add(fl, FEAT_TYPE_Q,     (char *)"s0Q", s0Q);
  feat_lib_add(fl, FEAT_TYPE_R,     (char *)"s0R", s0R);
  feat_lib_add(fl, FEAT_TYPE_S,     (char *)"s0S", s0S);
  feat_lib_add(fl, FEAT_TYPE_T,     (char *)"s0T", s0T);
  feat_lib_add(fl, FEAT_TYPE_U,     (char *)"s0U", s0U);
  feat_lib_add(fl, FEAT_TYPE_V,     (char *)"s0V", s0V);
  feat_lib_add(fl, FEAT_TYPE_W,     (char *)"s0W", s0W);
  feat_lib_add(fl, FEAT_TYPE_X,     (char *)"s0X", s0X);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"s0Y", s0Y);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"s0Z", s0Z);

  feat_lib_add(fl, FEAT_TYPE_INT_3,     (char *)"s0U1", s0U1);
  feat_lib_add(fl, FEAT_TYPE_INT,     (char *)"s0sgn", s0sgn);

  
  feat_lib_add(fl, FEAT_TYPE_FORM,  (char *)"s1f", s1f);
  feat_lib_add(fl, FEAT_TYPE_LEMMA, (char *)"s1l", s1l);
  feat_lib_add(fl, FEAT_TYPE_CPOS,  (char *)"s1c", s1c);
  feat_lib_add(fl, FEAT_TYPE_POS,   (char *)"s1p", s1p);
  feat_lib_add(fl, FEAT_TYPE_FEATS, (char *)"s1m", s1m);
  feat_lib_add(fl, FEAT_TYPE_STAG,  (char *)"s1s", s1s);
  /* feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"s1r", s1r); */
  feat_lib_add(fl, FEAT_TYPE_A,     (char *)"s1A", s1A);
  feat_lib_add(fl, FEAT_TYPE_B,     (char *)"s1B", s1B);
  feat_lib_add(fl, FEAT_TYPE_C,     (char *)"s1C", s1C);
  feat_lib_add(fl, FEAT_TYPE_D,     (char *)"s1D", s1D);
  feat_lib_add(fl, FEAT_TYPE_E,     (char *)"s1E", s1E);
  feat_lib_add(fl, FEAT_TYPE_F,     (char *)"s1F", s1F);
  feat_lib_add(fl, FEAT_TYPE_G,     (char *)"s1G", s1G);
  feat_lib_add(fl, FEAT_TYPE_H,     (char *)"s1H", s1H);
  feat_lib_add(fl, FEAT_TYPE_I,     (char *)"s1I", s1I);
  feat_lib_add(fl, FEAT_TYPE_J,     (char *)"s1J", s1J);
  feat_lib_add(fl, FEAT_TYPE_K,     (char *)"s1K", s1K);
  feat_lib_add(fl, FEAT_TYPE_L,     (char *)"s1L", s1L);
  feat_lib_add(fl, FEAT_TYPE_M,     (char *)"s1M", s1M);
  feat_lib_add(fl, FEAT_TYPE_N,     (char *)"s1N", s1N);
  feat_lib_add(fl, FEAT_TYPE_O,     (char *)"s1O", s1O);
  feat_lib_add(fl, FEAT_TYPE_P,     (char *)"s1P", s1P);
  feat_lib_add(fl, FEAT_TYPE_Q,     (char *)"s1Q", s1Q);
  feat_lib_add(fl, FEAT_TYPE_R,     (char *)"s1R", s1R);
  feat_lib_add(fl, FEAT_TYPE_S,     (char *)"s1S", s1S);
  feat_lib_add(fl, FEAT_TYPE_T,     (char *)"s1T", s1T);
  feat_lib_add(fl, FEAT_TYPE_U,     (char *)"s1U", s1U);
  feat_lib_add(fl, FEAT_TYPE_V,     (char *)"s1V", s1V);
  feat_lib_add(fl, FEAT_TYPE_W,     (char *)"s1W", s1W);
  feat_lib_add(fl, FEAT_TYPE_X,     (char *)"s1X", s1X);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"s1Y", s1Y);
  feat_lib_add(fl, FEAT_TYPE_Z,     (char *)"s1Z", s1Z);


  
  feat_lib_add(fl, FEAT_TYPE_FORM,  (char *)"s2f", s2f);
  feat_lib_add(fl, FEAT_TYPE_LEMMA, (char *)"s2l", s2l);
  feat_lib_add(fl, FEAT_TYPE_CPOS,  (char *)"s2c", s2c);
  feat_lib_add(fl, FEAT_TYPE_POS,   (char *)"s2p", s2p);
  feat_lib_add(fl, FEAT_TYPE_FEATS, (char *)"s2m", s2m);
  feat_lib_add(fl, FEAT_TYPE_STAG,  (char *)"s2s", s2s);
  /* feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"s2r", s2r); */
  feat_lib_add(fl, FEAT_TYPE_A,     (char *)"s2A", s2A);
  feat_lib_add(fl, FEAT_TYPE_B,     (char *)"s2B", s2B);
  feat_lib_add(fl, FEAT_TYPE_C,     (char *)"s2C", s2C);
  feat_lib_add(fl, FEAT_TYPE_D,     (char *)"s2D", s2D);
  feat_lib_add(fl, FEAT_TYPE_E,     (char *)"s2E", s2E);
  feat_lib_add(fl, FEAT_TYPE_F,     (char *)"s2F", s2F);
  feat_lib_add(fl, FEAT_TYPE_G,     (char *)"s2G", s2G);
  feat_lib_add(fl, FEAT_TYPE_H,     (char *)"s2H", s2H);
  feat_lib_add(fl, FEAT_TYPE_I,     (char *)"s2I", s2I);
  feat_lib_add(fl, FEAT_TYPE_J,     (char *)"s2J", s2J);
  feat_lib_add(fl, FEAT_TYPE_K,     (char *)"s2K", s2K);
  feat_lib_add(fl, FEAT_TYPE_L,     (char *)"s2L", s2L);
  feat_lib_add(fl, FEAT_TYPE_M,     (char *)"s2M", s2M);
  feat_lib_add(fl, FEAT_TYPE_N,     (char *)"s2N", s2N);
  feat_lib_add(fl, FEAT_TYPE_O,     (char *)"s2O", s2O);
  feat_lib_add(fl, FEAT_TYPE_P,     (char *)"s2P", s2P);
  feat_lib_add(fl, FEAT_TYPE_Q,     (char *)"s2Q", s2Q);
  feat_lib_add(fl, FEAT_TYPE_R,     (char *)"s2R", s2R);
  feat_lib_add(fl, FEAT_TYPE_S,     (char *)"s2S", s2S);
  feat_lib_add(fl, FEAT_TYPE_T,     (char *)"s2T", s2T);
  feat_lib_add(fl, FEAT_TYPE_U,     (char *)"s2U", s2U);
  feat_lib_add(fl, FEAT_TYPE_V,     (char *)"s2V", s2V);
  feat_lib_add(fl, FEAT_TYPE_W,     (char *)"s2W", s2W);
  feat_lib_add(fl, FEAT_TYPE_X,     (char *)"s2X", s2X);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"s2Y", s2Y);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"s2Z", s2Z);



  
  feat_lib_add(fl, FEAT_TYPE_FORM,  (char *)"s3f", s3f);
  feat_lib_add(fl, FEAT_TYPE_LEMMA, (char *)"s3l", s3l);
  feat_lib_add(fl, FEAT_TYPE_CPOS,  (char *)"s3c", s3c);
  feat_lib_add(fl, FEAT_TYPE_POS,   (char *)"s3p", s3p);
  feat_lib_add(fl, FEAT_TYPE_FEATS, (char *)"s3m", s3m);
  feat_lib_add(fl, FEAT_TYPE_STAG,  (char *)"s3s", s3s);
  /* feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"s3r", s3r); */
  feat_lib_add(fl, FEAT_TYPE_A,     (char *)"s3A", s3A);
  feat_lib_add(fl, FEAT_TYPE_B,     (char *)"s3B", s3B);
  feat_lib_add(fl, FEAT_TYPE_C,     (char *)"s3C", s3C);
  feat_lib_add(fl, FEAT_TYPE_D,     (char *)"s3D", s3D);
  feat_lib_add(fl, FEAT_TYPE_E,     (char *)"s3E", s3E);
  feat_lib_add(fl, FEAT_TYPE_F,     (char *)"s3F", s3F);
  feat_lib_add(fl, FEAT_TYPE_G,     (char *)"s3G", s3G);
  feat_lib_add(fl, FEAT_TYPE_H,     (char *)"s3H", s3H);
  feat_lib_add(fl, FEAT_TYPE_I,     (char *)"s3I", s3I);
  feat_lib_add(fl, FEAT_TYPE_J,     (char *)"s3J", s3J);
  feat_lib_add(fl, FEAT_TYPE_K,     (char *)"s3K", s3K);
  feat_lib_add(fl, FEAT_TYPE_L,     (char *)"s3L", s3L);
  feat_lib_add(fl, FEAT_TYPE_M,     (char *)"s3M", s3M);
  feat_lib_add(fl, FEAT_TYPE_N,     (char *)"s3N", s3N);
  feat_lib_add(fl, FEAT_TYPE_O,     (char *)"s3O", s3O);
  feat_lib_add(fl, FEAT_TYPE_P,     (char *)"s3P", s3P);
  feat_lib_add(fl, FEAT_TYPE_Q,     (char *)"s3Q", s3Q);
  feat_lib_add(fl, FEAT_TYPE_R,     (char *)"s3R", s3R);
  feat_lib_add(fl, FEAT_TYPE_S,     (char *)"s3S", s3S);
  feat_lib_add(fl, FEAT_TYPE_T,     (char *)"s3T", s3T);
  feat_lib_add(fl, FEAT_TYPE_U,     (char *)"s3U", s3U);
  feat_lib_add(fl, FEAT_TYPE_V,     (char *)"s3V", s3V);
  feat_lib_add(fl, FEAT_TYPE_W,     (char *)"s3W", s3W);
  feat_lib_add(fl, FEAT_TYPE_X,     (char *)"s3X", s3X);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"s3Y", s3Y);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"s3Z", s3Z);


  feat_lib_add(fl, FEAT_TYPE_FORM,  (char *)"b0f", b0f);
  feat_lib_add(fl, FEAT_TYPE_LEMMA, (char *)"b0l", b0l);
  feat_lib_add(fl, FEAT_TYPE_CPOS,  (char *)"b0c", b0c);
  feat_lib_add(fl, FEAT_TYPE_POS,   (char *)"b0p", b0p);
  feat_lib_add(fl, FEAT_TYPE_FEATS, (char *)"b0m", b0m);
  feat_lib_add(fl, FEAT_TYPE_STAG,  (char *)"b0s", b0s);
  /* feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"b0r", b0r); */
  feat_lib_add(fl, FEAT_TYPE_A,     (char *)"b0A", b0A);
  feat_lib_add(fl, FEAT_TYPE_B,     (char *)"b0B", b0B);
  feat_lib_add(fl, FEAT_TYPE_C,     (char *)"b0C", b0C);
  feat_lib_add(fl, FEAT_TYPE_D,     (char *)"b0D", b0D);
  feat_lib_add(fl, FEAT_TYPE_E,     (char *)"b0E", b0E);
  feat_lib_add(fl, FEAT_TYPE_F,     (char *)"b0F", b0F);
  feat_lib_add(fl, FEAT_TYPE_G,     (char *)"b0G", b0G);
  feat_lib_add(fl, FEAT_TYPE_H,     (char *)"b0H", b0H);
  feat_lib_add(fl, FEAT_TYPE_I,     (char *)"b0I", b0I);
  feat_lib_add(fl, FEAT_TYPE_J,     (char *)"b0J", b0J);
  feat_lib_add(fl, FEAT_TYPE_K,     (char *)"b0K", b0K);
  feat_lib_add(fl, FEAT_TYPE_L,     (char *)"b0L", b0L);
  feat_lib_add(fl, FEAT_TYPE_M,     (char *)"b0M", b0M);
  feat_lib_add(fl, FEAT_TYPE_N,     (char *)"b0N", b0N);
  feat_lib_add(fl, FEAT_TYPE_O,     (char *)"b0O", b0O);
  feat_lib_add(fl, FEAT_TYPE_P,     (char *)"b0P", b0P);
  feat_lib_add(fl, FEAT_TYPE_Q,     (char *)"b0Q", b0Q);
  feat_lib_add(fl, FEAT_TYPE_R,     (char *)"b0R", b0R);
  feat_lib_add(fl, FEAT_TYPE_S,     (char *)"b0S", b0S);
  feat_lib_add(fl, FEAT_TYPE_T,     (char *)"b0T", b0T);
  feat_lib_add(fl, FEAT_TYPE_U,     (char *)"b0U", b0U);
  feat_lib_add(fl, FEAT_TYPE_V,     (char *)"b0V", b0V);
  feat_lib_add(fl, FEAT_TYPE_W,     (char *)"b0W", b0W);
  feat_lib_add(fl, FEAT_TYPE_X,     (char *)"b0X", b0X);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"b0Y", b0Y);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"b0Z", b0Z);

  feat_lib_add(fl, FEAT_TYPE_INT_3,     (char *)"b0U1", b0U1);
  feat_lib_add(fl, FEAT_TYPE_INT,     (char *)"b0sgn", b0sgn);
  
  feat_lib_add(fl, FEAT_TYPE_FORM,  (char *)"b1f", b1f);
  feat_lib_add(fl, FEAT_TYPE_LEMMA, (char *)"b1l", b1l);
  feat_lib_add(fl, FEAT_TYPE_CPOS,  (char *)"b1c", b1c);
  feat_lib_add(fl, FEAT_TYPE_POS,   (char *)"b1p", b1p);
  feat_lib_add(fl, FEAT_TYPE_FEATS, (char *)"b1m", b1m);
  feat_lib_add(fl, FEAT_TYPE_STAG,  (char *)"b1s", b1s);
  /* feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"b1r", b1r); */
  feat_lib_add(fl, FEAT_TYPE_A,     (char *)"b1A", b1A);
  feat_lib_add(fl, FEAT_TYPE_B,     (char *)"b1B", b1B);
  feat_lib_add(fl, FEAT_TYPE_C,     (char *)"b1C", b1C);
  feat_lib_add(fl, FEAT_TYPE_D,     (char *)"b1D", b1D);
  feat_lib_add(fl, FEAT_TYPE_E,     (char *)"b1E", b1E);
  feat_lib_add(fl, FEAT_TYPE_F,     (char *)"b1F", b1F);
  feat_lib_add(fl, FEAT_TYPE_G,     (char *)"b1G", b1G);
  feat_lib_add(fl, FEAT_TYPE_H,     (char *)"b1H", b1H);
  feat_lib_add(fl, FEAT_TYPE_I,     (char *)"b1I", b1I);
  feat_lib_add(fl, FEAT_TYPE_J,     (char *)"b1J", b1J);
  feat_lib_add(fl, FEAT_TYPE_K,     (char *)"b1K", b1K);
  feat_lib_add(fl, FEAT_TYPE_L,     (char *)"b1L", b1L);
  feat_lib_add(fl, FEAT_TYPE_M,     (char *)"b1M", b1M);
  feat_lib_add(fl, FEAT_TYPE_N,     (char *)"b1N", b1N);
  feat_lib_add(fl, FEAT_TYPE_O,     (char *)"b1O", b1O);
  feat_lib_add(fl, FEAT_TYPE_P,     (char *)"b1P", b1P);
  feat_lib_add(fl, FEAT_TYPE_Q,     (char *)"b1Q", b1Q);
  feat_lib_add(fl, FEAT_TYPE_R,     (char *)"b1R", b1R);
  feat_lib_add(fl, FEAT_TYPE_S,     (char *)"b1S", b1S);
  feat_lib_add(fl, FEAT_TYPE_T,     (char *)"b1T", b1T);
  feat_lib_add(fl, FEAT_TYPE_U,     (char *)"b1U", b1U);
  feat_lib_add(fl, FEAT_TYPE_V,     (char *)"b1V", b1V);
  feat_lib_add(fl, FEAT_TYPE_W,     (char *)"b1W", b1W);
  feat_lib_add(fl, FEAT_TYPE_X,     (char *)"b1X", b1X);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"b1Y", b1Y);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"b1Z", b1Z);

  feat_lib_add(fl, FEAT_TYPE_INT_3,     (char *)"b1U1", b1U1);
  feat_lib_add(fl, FEAT_TYPE_INT,     (char *)"b1sgn", b1sgn);

  
  feat_lib_add(fl, FEAT_TYPE_FORM,  (char *)"b2f", b2f);
  feat_lib_add(fl, FEAT_TYPE_LEMMA, (char *)"b2l", b2l);
  feat_lib_add(fl, FEAT_TYPE_CPOS,  (char *)"b2c", b2c);
  feat_lib_add(fl, FEAT_TYPE_POS,   (char *)"b2p", b2p);
  feat_lib_add(fl, FEAT_TYPE_FEATS, (char *)"b2m", b2m);
  feat_lib_add(fl, FEAT_TYPE_STAG,  (char *)"b2s", b2s);
  /* feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"b2r", b2r); */
  feat_lib_add(fl, FEAT_TYPE_A,     (char *)"b2A", b2A);
  feat_lib_add(fl, FEAT_TYPE_B,     (char *)"b2B", b2B);
  feat_lib_add(fl, FEAT_TYPE_C,     (char *)"b2C", b2C);
  feat_lib_add(fl, FEAT_TYPE_D,     (char *)"b2D", b2D);
  feat_lib_add(fl, FEAT_TYPE_E,     (char *)"b2E", b2E);
  feat_lib_add(fl, FEAT_TYPE_F,     (char *)"b2F", b2F);
  feat_lib_add(fl, FEAT_TYPE_G,     (char *)"b2G", b2G);
  feat_lib_add(fl, FEAT_TYPE_H,     (char *)"b2H", b2H);
  feat_lib_add(fl, FEAT_TYPE_I,     (char *)"b2I", b2I);
  feat_lib_add(fl, FEAT_TYPE_J,     (char *)"b2J", b2J);
  feat_lib_add(fl, FEAT_TYPE_K,     (char *)"b2K", b2K);
  feat_lib_add(fl, FEAT_TYPE_L,     (char *)"b2L", b2L);
  feat_lib_add(fl, FEAT_TYPE_M,     (char *)"b2M", b2M);
  feat_lib_add(fl, FEAT_TYPE_N,     (char *)"b2N", b2N);
  feat_lib_add(fl, FEAT_TYPE_O,     (char *)"b2O", b2O);
  feat_lib_add(fl, FEAT_TYPE_P,     (char *)"b2P", b2P);
  feat_lib_add(fl, FEAT_TYPE_Q,     (char *)"b2Q", b2Q);
  feat_lib_add(fl, FEAT_TYPE_R,     (char *)"b2R", b2R);
  feat_lib_add(fl, FEAT_TYPE_S,     (char *)"b2S", b2S);
  feat_lib_add(fl, FEAT_TYPE_T,     (char *)"b2T", b2T);
  feat_lib_add(fl, FEAT_TYPE_U,     (char *)"b2U", b2U);
  feat_lib_add(fl, FEAT_TYPE_V,     (char *)"b2V", b2V);
  feat_lib_add(fl, FEAT_TYPE_W,     (char *)"b2W", b2W);
  feat_lib_add(fl, FEAT_TYPE_X,     (char *)"b2X", b2X);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"b2Y", b2Y);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"b2Z", b2Z);



  feat_lib_add(fl, FEAT_TYPE_FORM,  (char *)"b3f", b3f);
  feat_lib_add(fl, FEAT_TYPE_LEMMA, (char *)"b3l", b3l);
  feat_lib_add(fl, FEAT_TYPE_CPOS,  (char *)"b3c", b3c);
  feat_lib_add(fl, FEAT_TYPE_POS,   (char *)"b3p", b3p);
  feat_lib_add(fl, FEAT_TYPE_FEATS, (char *)"b3m", b3m);
  feat_lib_add(fl, FEAT_TYPE_STAG,  (char *)"b3s", b3s);
  /* feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"b3r", b3r); */
  feat_lib_add(fl, FEAT_TYPE_A,     (char *)"b3A", b3A);
  feat_lib_add(fl, FEAT_TYPE_B,     (char *)"b3B", b3B);
  feat_lib_add(fl, FEAT_TYPE_C,     (char *)"b3C", b3C);
  feat_lib_add(fl, FEAT_TYPE_D,     (char *)"b3D", b3D);
  feat_lib_add(fl, FEAT_TYPE_E,     (char *)"b3E", b3E);
  feat_lib_add(fl, FEAT_TYPE_F,     (char *)"b3F", b3F);
  feat_lib_add(fl, FEAT_TYPE_G,     (char *)"b3G", b3G);
  feat_lib_add(fl, FEAT_TYPE_H,     (char *)"b3H", b3H);
  feat_lib_add(fl, FEAT_TYPE_I,     (char *)"b3I", b3I);
  feat_lib_add(fl, FEAT_TYPE_J,     (char *)"b3J", b3J);
  feat_lib_add(fl, FEAT_TYPE_K,     (char *)"b3K", b3K);
  feat_lib_add(fl, FEAT_TYPE_L,     (char *)"b3L", b3L);
  feat_lib_add(fl, FEAT_TYPE_M,     (char *)"b3M", b3M);
  feat_lib_add(fl, FEAT_TYPE_N,     (char *)"b3N", b3N);
  feat_lib_add(fl, FEAT_TYPE_O,     (char *)"b3O", b3O);
  feat_lib_add(fl, FEAT_TYPE_P,     (char *)"b3P", b3P);
  feat_lib_add(fl, FEAT_TYPE_Q,     (char *)"b3Q", b3Q);
  feat_lib_add(fl, FEAT_TYPE_R,     (char *)"b3R", b3R);
  feat_lib_add(fl, FEAT_TYPE_S,     (char *)"b3S", b3S);
  feat_lib_add(fl, FEAT_TYPE_T,     (char *)"b3T", b3T);
  feat_lib_add(fl, FEAT_TYPE_U,     (char *)"b3U", b3U);
  feat_lib_add(fl, FEAT_TYPE_V,     (char *)"b3V", b3V);
  feat_lib_add(fl, FEAT_TYPE_W,     (char *)"b3W", b3W);
  feat_lib_add(fl, FEAT_TYPE_X,     (char *)"b3X", b3X);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"b3Y", b3Y);
  feat_lib_add(fl, FEAT_TYPE_Y,     (char *)"b3Z", b3Z);



  /* feat_lib_add(fl, FEAT_TYPE_LEMMA, (char *)"gs0l", gs0l); */
  /* feat_lib_add(fl, FEAT_TYPE_POS,   (char *)"gs0p", gs0p); */
  
  feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"ldep_s0r", ldep_s0r);
  feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"rdep_s0r", rdep_s0r);
  feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"ldep_b0r", ldep_b0r);
  feat_lib_add(fl, FEAT_TYPE_LABEL, (char *)"rdep_b0r", rdep_b0r);

  feat_lib_add(fl, FEAT_TYPE_INT_7, (char *)"ndep_b0", ndep_b0);
  feat_lib_add(fl, FEAT_TYPE_INT_7, (char *)"ndep_s0", ndep_s0);
  
  /* distance features */
  feat_lib_add(fl, FEAT_TYPE_INT_7, (char *)"dist_s0_b0", dist_s0_b0);

/* configurational features */

  feat_lib_add(fl, FEAT_TYPE_INT_8, (char *)"sh", sh);
  feat_lib_add(fl, FEAT_TYPE_INT_8, (char *)"bh", bh);
  feat_lib_add(fl, FEAT_TYPE_INT_8, (char *)"dh", dh);

  feat_lib_add(fl, FEAT_TYPE_TRANS, (char *)"t1", t1);
  feat_lib_add(fl, FEAT_TYPE_TRANS, (char *)"t2", t2);
  feat_lib_add(fl, FEAT_TYPE_TRANS, (char *)"t3", t3);
  feat_lib_add(fl, FEAT_TYPE_TRANS, (char *)"t4", t4);


  return fl;
}

