#include<stdio.h>
#include<stdlib.h>
#include"feat_vec.h"
#include"util.h"
#include "feat_types.h"
#include "feat_model.h"

void feat_vec_concat(feat_vec *fv1, feat_vec *fv2)
{
  int i;
  for(i=0; i < fv2->nb; i++)
    if(fv2->t[i] != -1)
      feat_vec_add(fv1, fv2->t[i]);
}


feat_vec *feat_vec_new(int size)
{
  feat_vec *fv = (feat_vec *)memalloc(sizeof(feat_vec));
  fv->size = size;
  fv->nb = 0;
  fv->t = (int *)memalloc(size * sizeof(int));
  return fv;
}

void feat_vec_free(feat_vec *fv)
{
  if(fv){
    if(fv->t)
      free(fv->t);
    free(fv);
  }
}

feat_vec *feat_vec_copy(feat_vec *fv)
{
  feat_vec *copy = feat_vec_new(fv->size);
  int i;
  for(i=0; i < fv->nb; i++){
    feat_vec_add(copy, fv->t[i]);
  }
  return copy;
}

int feat_vec_add(feat_vec *fv, int feat)
{
  if(fv->nb == fv->size -1){
    fv->size = 2 * (fv->size + 1);
    fv->t = (int *)realloc(fv->t, fv->size * sizeof(int));
  }
  fv->t[fv->nb] = feat;
  fv->nb++;
  return fv->nb;
}

void feat_vec_empty(feat_vec *fv)
{
  fv->nb = 0;
}

void feat_vec_print_string(feat_vec *fv, dico *dico_features)
{
  int i;
  for(i=0; i < fv->nb; i++){
    if(fv->t[i] != -1)
      printf("\t%s", dico_int2string(dico_features, fv->t[i]));
    else
      printf("\tNULL");
  }
  printf("\n");
}

void feat_vec_print(FILE *f, feat_vec *fv)
{
  int i;
  for(i=0; i < fv->nb; i++)
    fprintf(f, "\t%d", fv->t[i]);
  fprintf(f, "\n");
}

void feat_vec_print_dnn(FILE *f, feat_vec *fv, feat_model *fm, mcd *m)
{
  int i,j;
  feat_desc *fd;
  simple_feat_desc *sfd;
  int mcd_representation;
  int column;

  for(i=0; i < fm->nbelem; i++){
    fd = fm->array[i];
    sfd = fd->array[0];
    
    if((sfd->type <= FEAT_TYPE_INT_16) && (sfd->type >= FEAT_TYPE_INT_1)){
      int range = sfd->type - FEAT_TYPE_INT_0;
      for(j=0; j < range; j++){
	/* if(i + j != 0) fprintf(f, " "); */
	if(j == fv->t[i])
	  fprintf(f, "1 ");
	else
	  fprintf(f, "0 ");
      }
    }
    else{
      column = m->type2col[sfd->type];
      mcd_representation = m->representation[column];
      if(mcd_representation == MCD_REPRESENTATION_EMB){
	/* if(i != 0) fprintf(f, " "); */
	word_emb_print(f, m->word_emb_array[column], fv->t[i]);
      }
      else if(mcd_representation == MCD_REPRESENTATION_VOCAB){
	for(j=0; j < m->dico_array[column]->nbelem; j++){
	  /* if(i + j != 0) fprintf(f, " "); */
	  if(j == fv->t[i])
	    fprintf(f, "1 ");
	  else
	    fprintf(f, "0 ");
	}
      }
    }
  fprintf(f, "\n");

  }
  fprintf(f, "\n");
}

/* void feat_vec_fill_input_array_dnn(fann_type *input_array, feat_vec *fv, feat_model *fm, mcd *m) */
void feat_vec_fill_input_array_dnn(float *input_array, feat_vec *fv, feat_model *fm, mcd *m)
{
  int i,j;
  feat_desc *fd;
  simple_feat_desc *sfd;
  int mcd_representation;
  int current_index = 0;
  int column;

  for(i=0; i < fm->nbelem; i++){
    fd = fm->array[i];
    sfd = fd->array[0];

    if((sfd->type <= FEAT_TYPE_INT_16) && (sfd->type >= FEAT_TYPE_INT_1)){
      int range = sfd->type - FEAT_TYPE_INT_0;
      for(j=0; j < range; j++){
	if(j == fv->t[i])
	  input_array[current_index++] = 1;
	else
	  input_array[current_index++] = 0;
      }
    }
    else{
      column = m->type2col[sfd->type];
      mcd_representation = m->representation[column];
      if(mcd_representation == MCD_REPRESENTATION_EMB){
	current_index =  word_emb_fill_input_array_dnn(input_array, m->word_emb_array[column], fv->t[i], current_index);
      }
      else if(mcd_representation == MCD_REPRESENTATION_VOCAB){
	for(j=0; j < m->dico_array[column]->nbelem; j++){
	  if(j == fv->t[i])
	    input_array[current_index++] = 1;
	  else
	    input_array[current_index++] = 0;
	}
      }
    }
 
  }
}

